from argparse import ArgumentParser

from rich.panel import Panel
from rich.console import Console

from r3make.fetch import fetch_config, define_search_paths
from r3make.build import r3make_build
from r3make.config import parse_config
from r3make.version import YEAR, MINOR, PATCH

def main():
    console = Console()
    define_search_paths()
    parser = ArgumentParser(description="r3make - A lightweight C/C++ build tool")
    parser.add_argument('target', help="Specify the build target within the r3make configuration file")
    r3make_build(console, parse_config(fetch_config(), target=parser.parse_args().target))

if __name__ == "__main__":
    main()


import r3make.utils as utils
from rich.panel import Panel
from rich.console import Console

R3MAKE_DEFAULT_CONFIG:dict = {
    "c-instance": "GCC",
    "c-targets": {}
}

R3MAKE_DEFAULT_TARGET: dict = {
    "r3make": {
        "flags": {},
        "pre-build": {},
        "post-build": {}
    },
    "c-flags": [],
    "c-defines": [],
    "src-dirs": [],
    "src-files": [],
    "inc-dirs": [],
    "lib-links": {},
    "out-dir": "bin",
    "out-type": "exe",
    "out-name": "program"
}

def parse_config(config_path:str, target: str="debug") -> dict | None:
    if not utils.os.path.exists(config_path): return None

    with open(config_path, "rb") as f:
        config = utils.json.load(f)
        f.close()
    
    # parse the config and set default values
    pconfig = {key: config.get(key, R3MAKE_DEFAULT_CONFIG[key]) for key in R3MAKE_DEFAULT_CONFIG}
    
    if target not in pconfig["c-targets"]:
        Console().print(Panel(f"[bold red]r3make[/] Invalid compilation target: {target}", expand=False))
        return
    tconfig = pconfig["c-targets"][target]
    tconfig = {key: tconfig.get(key, R3MAKE_DEFAULT_TARGET[key]) for key in R3MAKE_DEFAULT_TARGET}
    tconfig["r3make"] = {k: tconfig["r3make"].get(k, {}) for k in R3MAKE_DEFAULT_TARGET["r3make"] if tconfig.get("r3make", None)}

    # parse compiler instace configuration
    tconfig["c-instance"] = pconfig["c-instance"]
    tconfig["c-flags"] = [ c_flag.strip() for c_flag in tconfig["c-flags"] ]
    tconfig["c-defines"] = [ c_define.strip() for c_define in tconfig["c-defines"] ]
    
    # parse source file configuration
    tconfig["src-dirs"] = [ utils.os_path(src_dir) for src_dir in tconfig["src-dirs"] ]
    tconfig["src-files"] = [ utils.os_path(src_file) for src_file in tconfig["src-files"] ]
    
    # parse include file configuration
    tconfig["inc-dirs"] = [ utils.os_path(src_dir) for src_dir in tconfig["inc-dirs"] ]
    
    # parse output configuration
    tconfig["out-dir"] = utils.os_path(tconfig["out-dir"])
    
    for lib, lib_path in tconfig["lib-links"].items():
        tconfig["lib-links"][lib] = utils.os_path(lib_path) if isinstance(lib_path, str) else None

    return tconfig



from rich.panel import Panel
from rich.console import Console

import r3make.utils as utils
from r3make.fetch import DEFAULT_SEARCH_PATHS

console = Console()
def _nofiles_command(config:dict, param=None) -> None:
    console.print(Panel(f"[bold cyan]nofiles[/] Removing O-Files", expand=False))
    utils.os.system(f"rmdir {config["out-dir"]}{utils.SEP}ofiles /s /q")
    console.print(Panel(f"[bold cyan]nofiles[/] Removed O-Files", expand=False))

def _gitdeps_command(config:dict, param=None) -> None:
    path = DEFAULT_SEARCH_PATHS["OS"][1] if utils.platform.system() == "Windows" else DEFAULT_SEARCH_PATHS["OS"][2]
    for dep in param:
        console.print(Panel(f"[bold cyan]gitdeps[/] Fetching Github Dependency ({dep})", expand=False))
        dep_name = dep.replace("/", utils.SEP).replace("\\", utils.SEP).split(utils.SEP)[1]
        try:
            cwd = utils.os.getcwd()
            # clone the dep
            if not utils.os.path.exists(f"{path}{utils.SEP}{dep_name}"):
                utils.os.mkdir(f"{path}{utils.SEP}{dep_name}")
                utils.os.system(f"git clone https://github.com/{dep}.git {path}{utils.SEP}{dep_name}")
            
            utils.os.chdir(f"{path}{utils.SEP}{dep_name}")
            
            # check if the .r3make directory exists
            if not utils.os.path.exists(".r3make"):
                console.print(Panel(f"[bold red]gitdeps[/] Dependency missing .r3make directory ({dep})", expand=False))
                utils.os.kill(utils.os.getpid(), utils.signal.SIGBREAK)

            # extract output info to verify the desired build artifact was generated
            with open(f"r3make", "rb") as f:
                config = utils.json.load(f)
                f.close()

            target = config["c-targets"].get(f"{dep_name}")
            if not target:
                console.print(Panel(f"[bold red]gitdeps[/] Dependency missing repository compile-target ({dep})", expand=False))
                utils.os.kill(utils.os.getpid(), utils.signal.SIGBREAK)
            
            out_dir = target.get("out-dir", False)
            out_type = target.get("out-type", False)
            out_name = target.get("out-name", False)

            if not all([out_dir, out_type, out_name]):
                console.print(Panel(f"[bold red]gitdeps[/] Dependency missing compile-target output configuration ({dep})", expand=False))
                utils.os.kill(utils.os.getpid(), utils.signal.SIGBREAK)

            # check if this dependency is already built
            for r, d, f in utils.os.walk(f"{path}{utils.SEP}{dep_name}"):
                if utils.os_path(r).endswith(utils.os_path(out_dir)):
                    utils.os.chdir(f"{utils.os_path(r)}")
                    break
            
            if utils.os.path.exists(f"{out_name}.{out_type}"):
                console.print(Panel(f"[bold green]gitdeps[/] Dependency Already built ({dep})", expand=False))
                utils.os.chdir(cwd)
                continue
            else:
                # chdir into the .r3make directory and build the repository target
                utils.os.chdir(f"{path}{utils.SEP}{dep_name}{utils.SEP}.r3make")
                utils.os.system(f"r3make {dep_name}")
                
                for r, d, f in utils.os.walk(f"{path}{utils.SEP}{dep_name}"):
                    if utils.os_path(r).endswith(utils.os_path(out_dir)):
                        utils.os.chdir(f"{utils.os_path(r)}")
                        break
            
                if utils.os.path.exists(f"{out_name}.{out_type}"):
                    console.print(Panel(f"[bold green]gitdeps[/] Dependency built ({dep})", expand=False))
                    utils.os.chdir(cwd)
                    continue
                else:
                    console.print(Panel(f"[bold red]gitdeps[/] Fetched but failed to build dependency ({dep})", expand=False))
                    utils.os.kill(utils.os.getpid(), utils.signal.SIGBREAK)
            utils.os.chdir(cwd)
            continue
        except (PermissionError):
            console.print(Panel(f"[bold red]gitdeps[/] Higher-Level Privilages Needed To Fetch Github Dependencies", expand=False))
            utils.os.kill(utils.os.getpid(), utils.signal.SIGBREAK)

    console.print(Panel(f"[bold cyan]gitdeps[/] Fetched Github Dependencies", expand=False))

CBUILD_COMMANDS:dict = {
    "nofiles": _nofiles_command,
    "gitdeps": _gitdeps_command,
}
